#include <iostream>
#include <cstdint>

#include <so_5/all.hpp>
#include <so_5_extra/mboxes/broadcast.hpp>

using namespace std;

using namespace so_5;

/*
 * The first part.
 *
 * Definition of low-level pipeline implementation details.
 */

#include "../common/pipeline_parts.ipp"

/*
 * The second part.
 *
 * Definition of messages to be processed by a pipeline and
 * the message processing code.
 */

#include "../common/application_logic.ipp"

int main()
{
	// Launch SObjectizer in a separate thread.
	wrapped_env_t sobj;

	/*
	 * The third part.
	 *
	 * Definition of message processing pipeline and imitation of
	 * several measures from a sensor.
	 */

	// Make a pipeline.
	auto pipeline = make_pipeline( sobj.environment(),
			stage(validation) | stage(conversion) | broadcast(
				stage(archiving),
				stage(distribution),
				stage(range_checking) | stage(alarm_detector{}) | broadcast(
					stage(alarm_initiator),
					stage( []( const alarm_detected & v ) {
							alarm_distribution( cerr, v );
						} )
					)
				) );

	// Send messages to a pipeline in a loop with 10ms delays.
	for( uint8_t i = 0; i < static_cast< uint8_t >(250); i += 10 )
	{
		send< raw_value >(
				pipeline,
				raw_measure{ 0, 0, i } );
		std::this_thread::sleep_for( chrono::milliseconds{10} );
	}
}

